@@ -1,7 +1,7 @@
 #! /usr/bin/env python
 ############################################################################
 # Copyright (C) 2002  Paul Rodger <paul@paulrodger.com>
-#           (C) 2006  Nikolaus Schulz <microschulz@web.de>
+#           (C) 2006,2007  Nikolaus Schulz <microschulz@web.de>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,7 +17,7 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ############################################################################
-# $Id: test_archivemail.py 150 2006-10-30 18:25:28Z nikosch $
+# $Id: test_archivemail.py 231 2007-11-07 00:57:44Z nikosch $
 """
 Unit-test archivemail using 'PyUnit'.
 
@@ -329,6 +329,83 @@ class TestIsTooOld(unittest.TestCase):
             assert(not archivemail.is_older_than_days(time_message=time_msg, 
                 max_days=1))
 
+########## archivemail.parse_imap_url() unit testing #################
+
+class TestParseIMAPUrl(unittest.TestCase): 
+    def setUp(self):
+        archivemail.options.quiet = 1
+        archivemail.options.verbose = 0
+        archivemail.options.pwfile = None
+        
+    urls_withoutpass = [
+            ('imaps://user@example.org@imap.example.org/upperbox/lowerbox',
+                ('user', None, 'example.org@imap.example.org',
+                'upperbox/lowerbox')), 
+            ('imaps://"user@example.org"@imap.example.org/upperbox/lowerbox',
+                ('user@example.org', None, 'imap.example.org',
+                'upperbox/lowerbox')), 
+            ('imaps://user@example.org"@imap.example.org/upperbox/lowerbox',
+                ('user', None, 'example.org"@imap.example.org',
+                'upperbox/lowerbox')), 
+            ('imaps://"user@example.org@imap.example.org/upperbox/lowerbox',
+                ('"user', None, 'example.org@imap.example.org',
+                'upperbox/lowerbox')), 
+            ('imaps://"us\\"er@example.org"@imap.example.org/upperbox/lowerbox',
+                ('us"er@example.org', None, 'imap.example.org',
+                'upperbox/lowerbox')), 
+            ('imaps://user\\@example.org@imap.example.org/upperbox/lowerbox',
+                ('user\\', None, 'example.org@imap.example.org',
+                'upperbox/lowerbox'))
+    ]
+    urls_withpass = [
+            ('imaps://user@example.org:passwd@imap.example.org/upperbox/lowerbox',
+                ('user@example.org', 'passwd', 'imap.example.org',
+                'upperbox/lowerbox'), 
+                ('user', None, 'example.org:passwd@imap.example.org',
+                'upperbox/lowerbox')), 
+            ('imaps://"user@example.org:passwd@imap.example.org/upperbox/lowerbox',
+                ('"user@example.org', "passwd", 'imap.example.org',
+                'upperbox/lowerbox'), 
+                ('"user', None, 'example.org:passwd@imap.example.org',
+                'upperbox/lowerbox')), 
+            ('imaps://u\\ser\\@example.org:"p@sswd"@imap.example.org/upperbox/lowerbox', 
+                ('u\\ser\\@example.org', 'p@sswd', 'imap.example.org',
+                'upperbox/lowerbox'),
+                ('u\\ser\\', None, 'example.org:"p@sswd"@imap.example.org',
+                'upperbox/lowerbox'))
+    ]
+    # These are invalid when the password's not stripped. 
+    urls_onlywithpass = [
+            ('imaps://"user@example.org":passwd@imap.example.org/upperbox/lowerbox',
+                ('user@example.org', "passwd", 'imap.example.org',
+                'upperbox/lowerbox'))
+    ]
+    def testUrlsWithoutPwfile(self):
+        """Parse test urls with --pwfile option unset. This parses a password in
+        the URL, if present."""
+        archivemail.options.pwfile = None
+        for mbstr in self.urls_withpass + self.urls_withoutpass:
+            url = mbstr[0][mbstr[0].find('://')+3:]
+            result = archivemail.parse_imap_url(url)
+            self.assertEqual(result, mbstr[1])
+
+    def testUrlsWithPwfile(self):
+        """Parse test urls with --pwfile set.  In this case the ':' character
+        loses its meaning as a delimiter."""
+        archivemail.options.pwfile = "whocares.txt"
+        for mbstr in self.urls_withpass: 
+            url = mbstr[0][mbstr[0].find('://')+3:]
+            result = archivemail.parse_imap_url(url)
+            self.assertEqual(result, mbstr[2])
+        for mbstr in self.urls_onlywithpass: 
+            url = mbstr[0][mbstr[0].find('://')+3:]
+            self.assertRaises(archivemail.UnexpectedError, 
+                    archivemail.parse_imap_url, url)
+
+    def tearDown(self): 
+        archivemail.options.quiet = 0
+        archivemail.options.verbose = 0
+        archivemail.options.pwfile = None
 
 ########## acceptance testing ###########
 
@@ -962,6 +1039,102 @@ class TestArchiveDelete(TestCaseInTempdir):
         super(TestArchiveDelete, self).tearDown()
 
 
+class TestArchiveCopy(TestCaseInTempdir):
+    """make sure the 'copy' option works"""
+    old_mbox = None
+    new_mbox = None
+    mbox_backup_name = None
+    mbox_name = None
+
+    def setUp(self):
+        super(TestArchiveCopy, self).setUp()
+        archivemail.options.quiet = 1
+        archivemail.options.copy_old_mail = 1
+
+    def testNew(self):
+        """archiving a new mailbox with the 'copy' option"""
+        for execute in ("package", "system"):
+            self.setUp()
+            self.mbox_name = make_mbox(messages=3, hours_old=(24 * 179))
+            self.mbox_backup_name = tempfile.mkstemp()[1]
+            shutil.copyfile(self.mbox_name, self.mbox_backup_name)
+            if execute == "package":
+                archivemail.archive(self.mbox_name)
+            elif execute == "system":
+                run = "./archivemail.py --copy --quiet %s" % self.mbox_name
+                self.assertEqual(os.system(run), 0)
+            else:
+                sys.exit(1)
+            assert(os.path.exists(self.mbox_name))
+            # mbox must not have changed: 
+            assert(filecmp.cmp(self.mbox_name, self.mbox_backup_name, shallow=0))
+            archive_name = self.mbox_name + "_archive.gz"
+            # There is no archive. 
+            assert(not os.path.exists(archive_name))
+            self.tearDown()
+
+    def testMixed(self):
+        """archiving a mixed mailbox with the 'copy' option"""
+        for execute in ("package", "system"):
+            self.setUp()
+            self.new_mbox = make_mbox(messages=3, hours_old=(24 * 179))
+            self.old_mbox = make_mbox(messages=3, hours_old=(24 * 181))
+            self.mbox_name = tempfile.mkstemp()[1]
+            shutil.copyfile(self.new_mbox, self.mbox_name)
+            append_file(self.old_mbox, self.mbox_name)
+            self.mbox_backup_name = tempfile.mkstemp()[1]
+            shutil.copyfile(self.mbox_name, self.mbox_backup_name)
+            if execute == "package":
+                archivemail.archive(self.mbox_name)
+            elif execute == "system":
+                run = "./archivemail.py --copy --quiet %s" % self.mbox_name
+                self.assertEqual(os.system(run), 0)
+            else:
+                sys.exit(1)
+            assert(os.path.exists(self.mbox_name))
+            # mbox must not have changed: 
+            assert(filecmp.cmp(self.mbox_backup_name, self.mbox_name, shallow=0))
+            # archive has the old messages.
+            archive_name = self.mbox_name + "_archive.gz"
+            assert(os.path.exists(archive_name))
+            self.assertEqual(os.system("gzip -d %s" % archive_name), 0)
+            archive_name = self.mbox_name + "_archive"
+            assert(os.path.exists(archive_name))
+            assert(filecmp.cmp(archive_name, self.old_mbox, shallow=0))
+            self.tearDown()
+
+    def testOld(self):
+        """archiving an old mailbox with the 'copy' option"""
+        for execute in ("package", "system"):
+            self.setUp()
+            self.mbox_name = make_mbox(messages=3, hours_old=(24 * 181))
+            self.mbox_backup_name = tempfile.mkstemp()[1]
+            shutil.copyfile(self.mbox_name, self.mbox_backup_name)
+            if execute == "package":
+                archivemail.archive(self.mbox_name)
+            elif execute == "system":
+                run = "./archivemail.py --copy --quiet %s" % self.mbox_name
+                self.assertEqual(os.system(run), 0)
+            else:
+                sys.exit(1)
+            assert(os.path.exists(self.mbox_name))
+            # mbox must not have changed: 
+            assert(filecmp.cmp(self.mbox_backup_name, self.mbox_name, shallow=0))
+            # archive has the old messages.
+            archive_name = self.mbox_name + "_archive.gz"
+            assert(os.path.exists(archive_name))
+            self.assertEqual(os.system("gzip -d %s" % archive_name), 0)
+            archive_name = self.mbox_name + "_archive"
+            assert(os.path.exists(archive_name))
+            assert(filecmp.cmp(archive_name, self.mbox_name, shallow=0))
+            self.tearDown()
+
+    def tearDown(self):
+        archivemail.options.copy_old_mail = 0
+        archivemail.options.quiet = 0
+        super(TestArchiveCopy, self).tearDown()
+
+
 class TestArchiveMboxFlagged(TestCaseInTempdir):
     """make sure the 'include_flagged' option works"""
     def setUp(self):
